# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: a3m/server/rpc/proto/a3m.proto
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import enum_type_wrapper

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor.FileDescriptor(
    name="a3m/server/rpc/proto/a3m.proto",
    package="a3m",
    syntax="proto3",
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_pb=b'\n\x1e\x61\x33m/server/rpc/proto/a3m.proto\x12\x03\x61\x33m"*\n\rSubmitRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x0b\n\x03url\x18\x02 \x01(\t"\x19\n\x0bSubmitReply\x12\n\n\x02id\x18\x01 \x01(\t"\x19\n\x0bReadRequest\x12\n\n\x02id\x18\x01 \x01(\t"T\n\tReadReply\x12"\n\x06status\x18\x01 \x01(\x0e\x32\x12.a3m.PackageStatus\x12\x0b\n\x03job\x18\x02 \x01(\t\x12\x16\n\x04jobs\x18\x03 \x03(\x0b\x32\x08.a3m.Job""\n\x10ListTasksRequest\x12\x0e\n\x06job_id\x18\x01 \x01(\t"*\n\x0eListTasksReply\x12\x18\n\x05tasks\x18\x01 \x03(\x0b\x32\t.a3m.Task"\xa1\x01\n\x03Job\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\r\n\x05group\x18\x03 \x01(\t\x12\x0f\n\x07link_id\x18\x04 \x01(\t\x12\x1f\n\x06status\x18\x05 \x01(\x0e\x32\x0f.a3m.Job.Status"?\n\x06Status\x12\x0b\n\x07UNKNOWN\x10\x00\x12\x0c\n\x08\x43OMPLETE\x10\x01\x12\x0e\n\nPROCESSING\x10\x02\x12\n\n\x06\x46\x41ILED\x10\x03"\x8e\x01\n\x04Task\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0f\n\x07\x66ile_id\x18\x02 \x01(\t\x12\x11\n\texit_code\x18\x03 \x01(\x05\x12\x10\n\x08\x66ilename\x18\x04 \x01(\t\x12\x11\n\texecution\x18\x05 \x01(\t\x12\x11\n\targuments\x18\x06 \x01(\t\x12\x0e\n\x06stdout\x18\x07 \x01(\t\x12\x0e\n\x06stderr\x18\x08 \x01(\t*G\n\rPackageStatus\x12\n\n\x06\x46\x41ILED\x10\x00\x12\x0c\n\x08REJECTED\x10\x01\x12\x0c\n\x08\x43OMPLETE\x10\x02\x12\x0e\n\nPROCESSING\x10\x03\x32\xa3\x01\n\x08Transfer\x12\x30\n\x06Submit\x12\x12.a3m.SubmitRequest\x1a\x10.a3m.SubmitReply"\x00\x12*\n\x04Read\x12\x10.a3m.ReadRequest\x1a\x0e.a3m.ReadReply"\x00\x12\x39\n\tListTasks\x12\x15.a3m.ListTasksRequest\x1a\x13.a3m.ListTasksReply"\x00\x62\x06proto3',
)

_PACKAGESTATUS = _descriptor.EnumDescriptor(
    name="PackageStatus",
    full_name="a3m.PackageStatus",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="FAILED",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="REJECTED",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="COMPLETE",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="PROCESSING",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=612,
    serialized_end=683,
)
_sym_db.RegisterEnumDescriptor(_PACKAGESTATUS)

PackageStatus = enum_type_wrapper.EnumTypeWrapper(_PACKAGESTATUS)
FAILED = 0
REJECTED = 1
COMPLETE = 2
PROCESSING = 3


_JOB_STATUS = _descriptor.EnumDescriptor(
    name="Status",
    full_name="a3m.Job.Status",
    filename=None,
    file=DESCRIPTOR,
    create_key=_descriptor._internal_create_key,
    values=[
        _descriptor.EnumValueDescriptor(
            name="UNKNOWN",
            index=0,
            number=0,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="COMPLETE",
            index=1,
            number=1,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="PROCESSING",
            index=2,
            number=2,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.EnumValueDescriptor(
            name="FAILED",
            index=3,
            number=3,
            serialized_options=None,
            type=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    containing_type=None,
    serialized_options=None,
    serialized_start=402,
    serialized_end=465,
)
_sym_db.RegisterEnumDescriptor(_JOB_STATUS)


_SUBMITREQUEST = _descriptor.Descriptor(
    name="SubmitRequest",
    full_name="a3m.SubmitRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="name",
            full_name="a3m.SubmitRequest.name",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="url",
            full_name="a3m.SubmitRequest.url",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=39,
    serialized_end=81,
)


_SUBMITREPLY = _descriptor.Descriptor(
    name="SubmitReply",
    full_name="a3m.SubmitReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="id",
            full_name="a3m.SubmitReply.id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=83,
    serialized_end=108,
)


_READREQUEST = _descriptor.Descriptor(
    name="ReadRequest",
    full_name="a3m.ReadRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="id",
            full_name="a3m.ReadRequest.id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=110,
    serialized_end=135,
)


_READREPLY = _descriptor.Descriptor(
    name="ReadReply",
    full_name="a3m.ReadReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="status",
            full_name="a3m.ReadReply.status",
            index=0,
            number=1,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="job",
            full_name="a3m.ReadReply.job",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="jobs",
            full_name="a3m.ReadReply.jobs",
            index=2,
            number=3,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=137,
    serialized_end=221,
)


_LISTTASKSREQUEST = _descriptor.Descriptor(
    name="ListTasksRequest",
    full_name="a3m.ListTasksRequest",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="job_id",
            full_name="a3m.ListTasksRequest.job_id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=223,
    serialized_end=257,
)


_LISTTASKSREPLY = _descriptor.Descriptor(
    name="ListTasksReply",
    full_name="a3m.ListTasksReply",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="tasks",
            full_name="a3m.ListTasksReply.tasks",
            index=0,
            number=1,
            type=11,
            cpp_type=10,
            label=3,
            has_default_value=False,
            default_value=[],
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=259,
    serialized_end=301,
)


_JOB = _descriptor.Descriptor(
    name="Job",
    full_name="a3m.Job",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="id",
            full_name="a3m.Job.id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="name",
            full_name="a3m.Job.name",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="group",
            full_name="a3m.Job.group",
            index=2,
            number=3,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="link_id",
            full_name="a3m.Job.link_id",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="status",
            full_name="a3m.Job.status",
            index=4,
            number=5,
            type=14,
            cpp_type=8,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[_JOB_STATUS],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=304,
    serialized_end=465,
)


_TASK = _descriptor.Descriptor(
    name="Task",
    full_name="a3m.Task",
    filename=None,
    file=DESCRIPTOR,
    containing_type=None,
    create_key=_descriptor._internal_create_key,
    fields=[
        _descriptor.FieldDescriptor(
            name="id",
            full_name="a3m.Task.id",
            index=0,
            number=1,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="file_id",
            full_name="a3m.Task.file_id",
            index=1,
            number=2,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="exit_code",
            full_name="a3m.Task.exit_code",
            index=2,
            number=3,
            type=5,
            cpp_type=1,
            label=1,
            has_default_value=False,
            default_value=0,
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="filename",
            full_name="a3m.Task.filename",
            index=3,
            number=4,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="execution",
            full_name="a3m.Task.execution",
            index=4,
            number=5,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="arguments",
            full_name="a3m.Task.arguments",
            index=5,
            number=6,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="stdout",
            full_name="a3m.Task.stdout",
            index=6,
            number=7,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.FieldDescriptor(
            name="stderr",
            full_name="a3m.Task.stderr",
            index=7,
            number=8,
            type=9,
            cpp_type=9,
            label=1,
            has_default_value=False,
            default_value=b"".decode("utf-8"),
            message_type=None,
            enum_type=None,
            containing_type=None,
            is_extension=False,
            extension_scope=None,
            serialized_options=None,
            file=DESCRIPTOR,
            create_key=_descriptor._internal_create_key,
        ),
    ],
    extensions=[],
    nested_types=[],
    enum_types=[],
    serialized_options=None,
    is_extendable=False,
    syntax="proto3",
    extension_ranges=[],
    oneofs=[],
    serialized_start=468,
    serialized_end=610,
)

_READREPLY.fields_by_name["status"].enum_type = _PACKAGESTATUS
_READREPLY.fields_by_name["jobs"].message_type = _JOB
_LISTTASKSREPLY.fields_by_name["tasks"].message_type = _TASK
_JOB.fields_by_name["status"].enum_type = _JOB_STATUS
_JOB_STATUS.containing_type = _JOB
DESCRIPTOR.message_types_by_name["SubmitRequest"] = _SUBMITREQUEST
DESCRIPTOR.message_types_by_name["SubmitReply"] = _SUBMITREPLY
DESCRIPTOR.message_types_by_name["ReadRequest"] = _READREQUEST
DESCRIPTOR.message_types_by_name["ReadReply"] = _READREPLY
DESCRIPTOR.message_types_by_name["ListTasksRequest"] = _LISTTASKSREQUEST
DESCRIPTOR.message_types_by_name["ListTasksReply"] = _LISTTASKSREPLY
DESCRIPTOR.message_types_by_name["Job"] = _JOB
DESCRIPTOR.message_types_by_name["Task"] = _TASK
DESCRIPTOR.enum_types_by_name["PackageStatus"] = _PACKAGESTATUS
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SubmitRequest = _reflection.GeneratedProtocolMessageType(
    "SubmitRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _SUBMITREQUEST,
        "__module__": "a3m.server.rpc.proto.a3m_pb2"
        # @@protoc_insertion_point(class_scope:a3m.SubmitRequest)
    },
)
_sym_db.RegisterMessage(SubmitRequest)

SubmitReply = _reflection.GeneratedProtocolMessageType(
    "SubmitReply",
    (_message.Message,),
    {
        "DESCRIPTOR": _SUBMITREPLY,
        "__module__": "a3m.server.rpc.proto.a3m_pb2"
        # @@protoc_insertion_point(class_scope:a3m.SubmitReply)
    },
)
_sym_db.RegisterMessage(SubmitReply)

ReadRequest = _reflection.GeneratedProtocolMessageType(
    "ReadRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _READREQUEST,
        "__module__": "a3m.server.rpc.proto.a3m_pb2"
        # @@protoc_insertion_point(class_scope:a3m.ReadRequest)
    },
)
_sym_db.RegisterMessage(ReadRequest)

ReadReply = _reflection.GeneratedProtocolMessageType(
    "ReadReply",
    (_message.Message,),
    {
        "DESCRIPTOR": _READREPLY,
        "__module__": "a3m.server.rpc.proto.a3m_pb2"
        # @@protoc_insertion_point(class_scope:a3m.ReadReply)
    },
)
_sym_db.RegisterMessage(ReadReply)

ListTasksRequest = _reflection.GeneratedProtocolMessageType(
    "ListTasksRequest",
    (_message.Message,),
    {
        "DESCRIPTOR": _LISTTASKSREQUEST,
        "__module__": "a3m.server.rpc.proto.a3m_pb2"
        # @@protoc_insertion_point(class_scope:a3m.ListTasksRequest)
    },
)
_sym_db.RegisterMessage(ListTasksRequest)

ListTasksReply = _reflection.GeneratedProtocolMessageType(
    "ListTasksReply",
    (_message.Message,),
    {
        "DESCRIPTOR": _LISTTASKSREPLY,
        "__module__": "a3m.server.rpc.proto.a3m_pb2"
        # @@protoc_insertion_point(class_scope:a3m.ListTasksReply)
    },
)
_sym_db.RegisterMessage(ListTasksReply)

Job = _reflection.GeneratedProtocolMessageType(
    "Job",
    (_message.Message,),
    {
        "DESCRIPTOR": _JOB,
        "__module__": "a3m.server.rpc.proto.a3m_pb2"
        # @@protoc_insertion_point(class_scope:a3m.Job)
    },
)
_sym_db.RegisterMessage(Job)

Task = _reflection.GeneratedProtocolMessageType(
    "Task",
    (_message.Message,),
    {
        "DESCRIPTOR": _TASK,
        "__module__": "a3m.server.rpc.proto.a3m_pb2"
        # @@protoc_insertion_point(class_scope:a3m.Task)
    },
)
_sym_db.RegisterMessage(Task)


_TRANSFER = _descriptor.ServiceDescriptor(
    name="Transfer",
    full_name="a3m.Transfer",
    file=DESCRIPTOR,
    index=0,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
    serialized_start=686,
    serialized_end=849,
    methods=[
        _descriptor.MethodDescriptor(
            name="Submit",
            full_name="a3m.Transfer.Submit",
            index=0,
            containing_service=None,
            input_type=_SUBMITREQUEST,
            output_type=_SUBMITREPLY,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="Read",
            full_name="a3m.Transfer.Read",
            index=1,
            containing_service=None,
            input_type=_READREQUEST,
            output_type=_READREPLY,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
        _descriptor.MethodDescriptor(
            name="ListTasks",
            full_name="a3m.Transfer.ListTasks",
            index=2,
            containing_service=None,
            input_type=_LISTTASKSREQUEST,
            output_type=_LISTTASKSREPLY,
            serialized_options=None,
            create_key=_descriptor._internal_create_key,
        ),
    ],
)
_sym_db.RegisterServiceDescriptor(_TRANSFER)

DESCRIPTOR.services_by_name["Transfer"] = _TRANSFER

# @@protoc_insertion_point(module_scope)
